;; For working with RSS feeds
(ns rss-analyzer.rss
  (:require [clojure.xml :as xml]
            [rss-analyzer.utils :refer :all])
  (:gen-class))

(def RSS-PARTS
  "List of RSS tags that we want to use"
  ;; Todo: surround interactions with nil checks as RSS has only
  ;; title, link and description mandatory
  #{:title :guid :description :link :content})

(defn get-unread-part
  ;; we will need more dynamic version as we may want to also include
  ;; custom comparsions
  "Gets only unread part of RSS, splits by GUID or by specified :key
  which gets compared to given val"
  ([feed last-val] (get-unread-part feed :guid last-val))
  ([feed guid-key last-val]
   (split-with #(not= (first (%1 guid-key)) ;; (first) because guid is in vector
                   last-val) feed)))

(defn rss-item-to-map
  "Converts item's vector of contents to single map"
  ([item-vect] (rss-item-to-map item-vect (transient {})))
  ([[current & remaining] ret]
   (if current
     (if (some #{(current :tag)} RSS-PARTS)
       (recur
        remaining
        (conj! ret {(current :tag) (current :content)}))
       (recur
        remaining
        ret))
     (persistent! ret))))

(defn normalize-RSS-feed
  "Transforms raw RSS feed (list of maps and vectors) into vector of feed maps
  to make feed easier to work with."
  ;; By default, the item has a vector of tags and the order is not guaranteed
  ;; This makes it hard to be really usefull
  ([stripped-feed] (normalize-RSS-feed stripped-feed (transient [])))
  ([[current-item & feed-rest] ret]
   (if feed-rest
     (recur feed-rest (conj! ret (rss-item-to-map (current-item :content))))
     (persistent! ret))))

(defn download-and-trim-feed
  "Returns parsed feed stripped of uneccesarry tags"
  [feed-url]
  (try
    (let [feed (as-> feed-url arg
                 (xml/parse arg)
                 (arg :content)
                 (first arg)
                 (arg :content))
          ;;returning only second part - data; as the meta part is mostly useless and
          ;;can be derived from data
          [meta data] (split-with
                       #(not= (%1 :tag) :item) feed)] data)
    (catch Exception e
      ;;We can get HTTP errors (non 200) and parsing erros here
      ;;so just pretty much adding more info for the user
      ;;Todo: additional catches to be more helpfull
      (throw (ex-info (ex-message e)
                      {:program-part "RSS feed download"
                       :solution "use different computer/java/internet or something"})))))
